

import copy
import json

from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run



_parserColonKVP = ParseAtFirstDelimiter(delimiter="=", valueCanBeWrappedInDoubleQuotes=True)



#
# Returns:
#
#	{
#		"devtree": [
#			{
#				"children": [
#					{
#						"dev": "/dev/sda1",
#						"fstype": "ext4",
#						"label": null,
#						"mountpoint": "/",
#						"name": "sda1",
#						"uuid": "94933739-97e5-47e8-b7e8-ab8b7ed3f2a7"
#					}
#				],
#				"dev": "/dev/sda",
#				"fstype": null,
#				"label": null,
#				"mountpoint": null,
#				"name": "sda",
#				"uuid": null
#			},
#			{
#				"dev": "/dev/sr0",
#				"fstype": "iso9660",
#				"label": "YYYYYY",
#				"mountpoint": "/media/xxxxxxxx/YYYYYY",
#				"name": "sr0",
#				"uuid": "2001-10-05-02-59-13-00"
#			}
#		],
#		"mountpoints": {
#			"/": {
#				"dev": "/dev/sda1",
#				"fstype": "ext4",
#				"label": null,
#				"mountpoint": "/",
#				"name": "sda1",
#				"uuid": "94933739-97e5-47e8-b7e8-ab8b7ed3f2a7"
#			},
#			"/media/xxxxxxxx/YYYYYY": {
#				"dev": "/dev/sr0",
#				"fstype": "iso9660",
#				"label": "YYYYYY",
#				"mountPoint": "/media/xxxxxxxx/YYYYYY",
#				"name": "sr0",
#				"uuid": "2001-10-05-02-59-13-00"
#			}
#		}
#	}
#
def parse_lsblk(stdout:str, stderr:str, exitcode:int) -> dict:

	"""
	{
		"blockdevices": [
			{"name": "sda", "fstype": null, "label": null, "uuid": null, "mountpoint": null,
				"children": [
				{"name": "sda1", "fstype": "ext4", "label": null, "uuid": "94933739-97e5-47e8-b7e8-ab8b7ed3f2a7", "mountpoint": "/"}
				]
			},
			{"name": "sr0", "fstype": "iso9660", "label": "YYYYYY", "uuid": "2001-10-05-02-59-13-00", "mountpoint": "/media/xxxxxxxx/YYYYYY"}
		]
	}
	"""

	if exitcode != 0:
		raise Exception()

	ret = json.loads(stdout.strip())["blockdevices"]
	mountPointMap = {}
	for jBlockDevice in ret:
		__postproces_lsblk_dev(jBlockDevice, mountPointMap)

	return {
		"deviceTree": ret,
		"mountPoints": mountPointMap,
	}
#

def __postproces_lsblk_dev(j, mountPointMap):
	j["dev"] = j["name"]
	if "children" in j:
		for j2 in j["children"]:
			__postproces_lsblk_dev(j2, mountPointMap)
	if j["mountpoint"]:
		j2 = copy.deepcopy(j)
		if "children" in j2:
			del j2["children"]
		mountPointMap[j2["mountpoint"]] = j2
#



#
# Returns:
#
#	{
#		"devtree": [
#			{
#				"children": [
#					{
#						"dev": "/dev/sda1",
#						"fstype": "ext4",
#						"label": null,
#						"mountpoint": "/",
#						"name": "sda1",
#						"uuid": "94933739-97e5-47e8-b7e8-ab8b7ed3f2a7"
#					}
#				],
#				"dev": "/dev/sda",
#				"fstype": null,
#				"label": null,
#				"mountpoint": null,
#				"name": "sda",
#				"uuid": null
#			},
#			{
#				"dev": "/dev/sr0",
#				"fstype": "iso9660",
#				"label": "YYYYYY",
#				"mountpoint": "/media/xxxxxxxx/YYYYYY",
#				"name": "sr0",
#				"uuid": "2001-10-05-02-59-13-00"
#			}
#		],
#		"mountpoints": {
#			"/": {
#				"dev": "/dev/sda1",
#				"fstype": "ext4",
#				"label": null,
#				"mountpoint": "/",
#				"name": "sda1",
#				"uuid": "94933739-97e5-47e8-b7e8-ab8b7ed3f2a7"
#			},
#			"/media/xxxxxxxx/YYYYYY": {
#				"dev": "/dev/sr0",
#				"fstype": "iso9660",
#				"label": "YYYYYY",
#				"mountPoint": "/media/xxxxxxxx/YYYYYY",
#				"name": "sr0",
#				"uuid": "2001-10-05-02-59-13-00"
#			}
#		}
#	}
#
@cacheCalls(seconds=3, dependArgs=[0])
def get_lsblk(c = None) -> dict:
	stdout, stderr, exitcode = run(c, "/bin/lsblk -bJpO")
	return parse_lsblk(stdout, stderr, exitcode)
#















import re

from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run

from .get_etc_os_release import get_etc_os_release
from .get_vcgencmd import get_vcgencmd_get_config




#
# Returns:
#	{
#		"count": 4,
#		"cpus": [
#			{
#				"freq_max": 3900,
#				"freq_min": 800
#			},
#			{
#				"freq_max": 3900,
#				"freq_min": 800
#			},
#			{
#				"freq_max": 3900,
#				"freq_min": 800
#			},
#			{
#				"freq_max": 3900,
#				"freq_min": 800
#			}
#			],
#		"freq_max": 3900,
#		"freq_min": 800
#	}
#
@cacheCalls(seconds=3, dependArgs=[0])
def get_cpu_info(c = None) -> dict:
	os_release = get_etc_os_release(c)

	ret = {}

	stdout, _, _ = run(c, "/bin/ls /sys/devices/system/cpu/")
	cpus = []
	for x in stdout.strip().split():
		if x:
			if re.match(r"^cpu(\d+)$", x):
				cpus.append(x)

	ret = {
		"count": len(cpus),
		"cpus": [],
	}

	if os_release["distribution"] == "raspbian":
		# Raspian Linux
		_cfg_result = get_vcgencmd_get_config(c)
		ret["cpus"].append({
			"freq_min": _cfg_result["cpu"]["freq_min"],
			"freq_max": _cfg_result["cpu"]["freq_max"],
		})

	else:
		# other Linux
		for cpu in cpus:
			n = int(cpu[3:])
			stdoutMin, _, _ = run(c, "cat /sys/devices/system/cpu/cpufreq/policy" + str(n) + "/cpuinfo_min_freq")
			stdoutMax, _, _ = run(c, "cat /sys/devices/system/cpu/cpufreq/policy" + str(n) + "/cpuinfo_max_freq")
			freqMin = int(stdoutMin.strip()) // 1000
			freqMax = int(stdoutMax.strip()) // 1000
			ret["cpus"].append({
				"freq_min": freqMin,
				"freq_max": freqMax,
			})

	totalFreqMin = 999999999999999
	totalFreqMax = 0
	for cpu in ret["cpus"]:
		freqMin = cpu["freq_min"]
		freqMax = cpu["freq_max"]
		if freqMax > totalFreqMax:
			totalFreqMax = freqMax
		if freqMin < totalFreqMin:
			totalFreqMin = freqMin
	ret["freq_min"] = totalFreqMin
	ret["freq_max"] = totalFreqMax

	return ret
#













import datetime
import pytz

from jk_cachefunccalls import cacheCalls

from .parsing_utils import *
from .invoke_utils import run



#
#
#
def parse_date_as_datetime(stdout:str, stderr:str, exitcode:int, utc:bool = False) -> datetime.datetime:

	"""
	2019-09-01-23-03-53-549067419
	"""

	if exitcode != 0:
		raise Exception()

	sYear, sMonth, sDay, sHours, sMinutes, sSeconds, sNanoSeconds = stdout.strip().split("-")

	if utc:
		return datetime.datetime(
			int(sYear), int(sMonth), int(sDay), int(sHours), int (sMinutes), int(sSeconds), int(sNanoSeconds)//1000,
			pytz.UTC)
	else:
		return datetime.datetime(
			int(sYear), int(sMonth), int(sDay), int(sHours), int (sMinutes), int(sSeconds), int(sNanoSeconds)//1000)
#



#
# Get the date as reported by the (possibly remote) operating system.
#
@cacheCalls(seconds=3, dependArgs=[0, 1])
def get_date(c = None, utc:bool = True) -> dict:
	stdout, stderr, exitcode = run(c, "/bin/date " + ("-u" if utc else "") + " +'%Y-%m-%d-%H-%M-%S-%N'")
	d = parse_date_as_datetime(stdout, stderr, exitcode, utc)
	return {
		"year": d.year,
		"month": d.month,
		"day": d.day,
		"hour": d.hour,
		"minute": d.minute,
		"second": d.second,
		"millisecond": d.microsecond // 1000,
	}
#



#
# Get the date as reported by the (possibly remote) operating system.
#
@cacheCalls(seconds=3, dependArgs=[0, 1])
def get_date_as_datetime(c = None, utc:bool = True) -> datetime.datetime:
	stdout, stderr, exitcode = run(c, "/bin/date " + ("-u" if utc else "") + " +'%Y-%m-%d-%H-%M-%S-%N'")
	return parse_date_as_datetime(stdout, stderr, exitcode, utc)
#




















import re

from jk_cachefunccalls import cacheCalls
#import jk_json

from .parsing_utils import *
from .invoke_utils import run



_parserEqKVPs = ParseAtFirstDelimiter(delimiter="=", valueCanBeWrappedInDoubleQuotes=False)
_parserColonKVPs = ParseAtFirstDelimiter(delimiter=":", valueCanBeWrappedInDoubleQuotes=False)



#
# Returns:
#
#	{
#		"cpu": {
#			"freq_current": 500,
#			"freq_max": 1500,
#			"freq_min": 200
#		},
#		"gpu": {
#			"freq_max": 500,
#			"freq_min": 250
#		},
#		"ram": {
#			"total_mbytes": 4096
#		}
#	}
#
def _parse_vcgencmd_get_config(stdout:str, stderr:str, exitcode:int, outData:dict = None) -> dict:
	if exitcode != 0:
		raise Exception()

	data = _parserEqKVPs.parseLines(stdout.strip().split("\n"))
	#jk_json.prettyPrint(data)

	if outData is None:
		outData = {}

	if not "cpu" in outData:
		outData["cpu"] = {}

	outData["cpu"]["freq_max"] = int(data["arm_freq"])
	outData["cpu"]["freq_min"] = int(data["core_freq_min"])
	outData["cpu"]["freq_current"] = int(data["core_freq"])

	if not "gpu" in outData:
		outData["gpu"] = {}

	outData["gpu"]["freq_max"] = int(data["gpu_freq"])
	outData["gpu"]["freq_min"] = int(data["gpu_freq_min"])

	if not "ram" in outData:
		outData["ram"] = {}

	outData["ram"]["total_mbytes"] = int(data["total_mem"])

	return outData
#



#
# Returns:
#
#	{
#		"cpu": {
#			"freq_current": 500,
#			"freq_max": 1500,
#			"freq_min": 200
#		},
#		"gpu": {
#			"freq_max": 500,
#			"freq_min": 250
#		},
#		"ram": {
#			"total_mbytes": 4096
#		}
#	}
#
@cacheCalls(seconds=3, dependArgs=[0])
def get_vcgencmd_get_config(c = None, outData:dict = None) -> dict:
	stdout, stderr, exitcode = run(c, "/usr/bin/vcgencmd get_config int")
	return _parse_vcgencmd_get_config(stdout, stderr, exitcode, outData)
#



def _parse_vcgencmd_measure_volts(stdout:str, stderr:str, exitcode:int, key:str, outData:dict = None) -> dict:
	if exitcode != 0:
		raise Exception()

	data = _parserEqKVPs.parseLines(stdout.strip().split("\n"))
	#jk_json.prettyPrint(data)

	if outData is None:
		outData = {}

	if key not in outData:
		outData[key] = {}

	assert data["volt"].endswith("V")
	outData[key]["volt"] = float(data["volt"][:-1])

	return outData
#



@cacheCalls(seconds=3, dependArgs=[0])
def get_vcgencmd_measure_volts(c = None, outData:dict = None) -> dict:
	stdout, stderr, exitcode = run(c, "/usr/bin/vcgencmd measure_volts core")
	outData = _parse_vcgencmd_measure_volts(stdout, stderr, exitcode, "cpu", outData)
	stdout, stderr, exitcode = run(c, "/usr/bin/vcgencmd measure_volts sdram_c")
	outData = _parse_vcgencmd_measure_volts(stdout, stderr, exitcode, "ram", outData)
	return outData
#



def _parse_vcgencmd_measure_temp(stdout:str, stderr:str, exitcode:int, outData:dict = None) -> dict:
	if exitcode != 0:
		raise Exception()

	data = _parserEqKVPs.parseLines(stdout.strip().split("\n"))
	#jk_json.prettyPrint(data)

	if outData is None:
		outData = {}

	if not "cpu" in outData:
		outData["cpu"] = {}

	assert data["temp"].endswith("'C")
	outData["cpu"]["temp"] = float(data["temp"][:-2])

	return outData
#



@cacheCalls(seconds=3, dependArgs=[0])
def get_vcgencmd_measure_temp(c = None, outData:dict = None) -> dict:
	stdout, stderr, exitcode = run(c, "/usr/bin/vcgencmd measure_temp")
	return _parse_vcgencmd_measure_temp(stdout, stderr, exitcode, outData)
#



def _parse_vcgencmd_get_mem(stdout:str, stderr:str, exitcode:int, key:str, outData:dict = None) -> dict:
	if exitcode != 0:
		raise Exception()

	data = _parserEqKVPs.parseLines(stdout.strip().split("\n"))
	#jk_json.prettyPrint(data)
	assert len(data) == 1
	for v in data.values():
		assert v.endswith("M")
		value = int(v[:-1])
		break

	if outData is None:
		outData = {}

	if not "ram" in outData:
		outData["ram"] = {}

	outData["ram"][key] = value

	return outData
#



@cacheCalls(seconds=3, dependArgs=[0])
def get_vcgencmd_get_mem(c = None, outData:dict = None) -> dict:
	stdout, stderr, exitcode = run(c, "/usr/bin/vcgencmd get_mem arm")
	outData = _parse_vcgencmd_get_mem(stdout, stderr, exitcode, "system_mbytes", outData)
	stdout, stderr, exitcode = run(c, "/usr/bin/vcgencmd get_mem gpu")
	outData = _parse_vcgencmd_get_mem(stdout, stderr, exitcode, "gpu_mbytes", outData)
	return outData
#



def _parse_vcgencmd_display_power(stdout:str, stderr:str, exitcode:int, outData:dict = None) -> dict:
	if exitcode != 0:
		raise Exception()

	data = _parserEqKVPs.parseLines(stdout.strip().split("\n"))
	#jk_json.prettyPrint(data)
	assert len(data) == 1
	for v in data.values():
		value = int(v)
		break

	if outData is None:
		outData = {}

	if "display" not in outData:
		outData["display"] = {}

	outData["display"]["on"] = value > 0

	return outData
#



@cacheCalls(seconds=3, dependArgs=[0])
def get_vcgencmd_display_power(c = None, outData:dict = None) -> dict:
	stdout, stderr, exitcode = run(c, "/usr/bin/vcgencmd display_power")
	outData = _parse_vcgencmd_display_power(stdout, stderr, exitcode, outData)
	return outData
#



#
# Returns:
#
#	{
#		"cpu": {
#			"freq_current": 500,
#			"freq_max": 1500,
#			"freq_min": 200,
#			"temp": 36.0,
#			"volt": 0.8625
#		},
#		"display": {
#			"on": false
#		},
#		"gpu": {
#			"freq_max": 500,
#			"freq_min": 250
#		},
#		"ram": {
#			"gpu_mbytes": 76,
#			"system_mbytes": 948,
#			"total_mbytes": 4096,
#			"volt": 1.1
#		}
#	}
#
def get_vcgencmd(c = None, _ignoreCache:bool = False) -> dict:
	ret = {}
	get_vcgencmd_get_config(c, ret, _ignoreCache=_ignoreCache)
	get_vcgencmd_measure_volts(c, ret, _ignoreCache=_ignoreCache)
	get_vcgencmd_measure_temp(c, ret, _ignoreCache=_ignoreCache)
	get_vcgencmd_get_mem(c, ret, _ignoreCache=_ignoreCache)
	get_vcgencmd_display_power(c, ret, _ignoreCache=_ignoreCache)
	return ret
#












import sys
import re

from jk_cachefunccalls import cacheCalls
#import jk_json

from .parsing_utils import *
from .invoke_utils import run
from .get_net_info import _get_network_file_list





#
# Returns:
#	{
#		"bitrates": [
#			{
#				"unit": "Mb/s",
#				"values": [
#					1.0,
#					2.0,
#					5.5,
#					11.0,
#					6.0,
#					9.0,
#					12.0,
#					18.0,
#					24.0,
#					36.0,
#					48.0,
#					54.0
#				]
#			}
#		],
#		"encrypted": true,
#		"encryption": {
#			"IEEE 802.11i/WPA2 Version 1": {
#				"Authentication Suites (1)": "PSK",
#				"Group Cipher": "CCMP",
#				"Pairwise Ciphers (1)": "CCMP"
#			}
#		},
#		"essid": "HotelChernoMore",
#		"frequency": {
#			"channel": 11,
#			"unit": "GHz",
#			"value": 2.462
#		},
#		"mac": "F0:9F:C2:91:E3:1D",
#		"signal": {
#			"quality_cur": 52,
#			"quality_max": 70,
#			"signal_level": {
#				"unit": "dBm",
#				"value": -58
#			}
#		}
#	}
#
def __parse_single_wifi_cell(interface:str, radioCellLines:list, radioCellLinesIE:list):
	ret = {
		"essid": None
	}

	for line in radioCellLines:

		# "Frequency:2.412 GHz (Channel 1)"
		m = re.match(r"^Frequency:([\d\.]+)\s+([MGT]Hz)\s+\(Channel\s+(\d+)\)$", line)
		if m:
			groups = m.groups()
			ret["frequency"] = {
				"value": float(groups[0]),
				"unit": groups[1],
				"channel": int(groups[2]),
			}
			continue

		# "Address: E4:8D:8C:6D:C3:F5"
		m = re.match(r"^Address:\s+(.*)$", line)
		if m:
			groups = m.groups()
			ret["mac"] = groups[0]

		# "Channel:1"

		# "Quality=63/70  Signal level=-47 dBm  "
		m = re.match(r"^Quality=(\d+)/(\d+)\s+Signal\s+level=([-\d]+)\s+dBm\s+$", line)
		if m:
			groups = m.groups()
			ret["signal"] = {
				"quality_cur": int(groups[0]),
				"quality_max": int(groups[1]),
				"signal_level": {
					"value": int(groups[2]),
					"unit": "dBm",
				}
			}

		# "Encryption key:on"
		m = re.match(r"^Encryption\s+key:(.*)$", line)
		if m:
			groups = m.groups()
			ret["encrypted"] = groups[0] == "on"

		# "ESSID:\"ConfHall 1,2\""
		m = re.match(r"^ESSID:\"(.+)\"$", line)
		if m:
			groups = m.groups()
			ret["essid"] = groups[0]

		# "Bit Rates:1 Mb/s; 2 Mb/s; 5.5 Mb/s; 11 Mb/s; 6 Mb/s	  9 Mb/s; 12 Mb/s; 18 Mb/s"
		# "Bit Rates:24 Mb/s; 36 Mb/s; 48 Mb/s; 54 Mb/s"
		m = re.match(r"^Bit Rates:(.*)$", line)
		if m:
			bitrates = ret.get("bitrates")
			if bitrates is None:
				bitrates = []
				ret["bitrates"] = bitrates

			# "1 Mb/s; 2 Mb/s; 5.5 Mb/s; 11 Mb/s; 6 Mb/s	  9 Mb/s; 12 Mb/s; 18 Mb/s"
			items = re.split(";|\s\s\s", str(m.groups()[0]))
			for item in items:
				item = item.strip()
				if item:
					# "12 MB/s"
					m = re.match(r"^\s*([\.\d]+)\s+([GM]b/s)\s*$", item)
					if m:
						groups = m.groups()
						bitrates.append({
							"value": float(groups[0]),
							"unit": groups[1],
						})
					else:
						raise Exception("Failed to match: " + repr(item))


		# "Mode:Master"

		# "Extra:tsf=000000411f6f43d8"

		# "Extra: Last beacon: 4364ms ago"
		#m = re.match(r"^$", line)
		#if m:
		#	groups = m.groups()
		#	ret["mac"] = groups[0]

	ret["bitrates"] = simplifyValueList(ret["bitrates"])

	# now process the rest of the data

	radioCellDictIE = {}
	for key, values in groupLinesByLeadingSpace(radioCellLinesIE).items():
		if key.startswith("IE: "):
			key = key[4:]
			if not key.startswith("Unknown: "):
				radioCellDictIE[key] = values
		else:
			raise Exception("Unexpected key: " + repr(key))

	"""
	{
	"IEEE 802.11i/WPA2 Version 1": [
		"Group Cipher : TKIP",
		"Pairwise Ciphers (2) : CCMP TKIP",
		"Authentication Suites (1) : PSK"
	],
	"WPA Version 1": [
		"Group Cipher : TKIP",
		"Pairwise Ciphers (2) : CCMP TKIP",
		"Authentication Suites (1) : PSK"
	]
	}
	"""

	encryptionData = {}
	for key, lines in radioCellDictIE.items():
		encryptionSchemaData = {}
		for line in lines:
			pos = line.find(" : ")
			if pos < 0:
				raise Exception("Unexpected line: " + repr(line))
			encryptionSchemaData[line[:pos].strip()] = line[pos+3:].strip()
		encryptionData[key] = encryptionSchemaData

	ret["encryption"] = encryptionData

	return ret
#

#
# Returns:
#	[
#		"0": {
#			"bitrates": [
#				{
#					"unit": "Mb/s",
#					"values": [
#						1.0,
#						2.0,
#						5.5,
#						11.0,
#						6.0,
#						9.0,
#						12.0,
#						18.0,
#						24.0,
#						36.0,
#						48.0,
#						54.0
#					]
#				}
#			],
#			"encrypted": true,
#			"encryption": {
#				"IEEE 802.11i/WPA2 Version 1": {
#					"Authentication Suites (1)": "PSK",
#					"Group Cipher": "CCMP",
#					"Pairwise Ciphers (1)": "CCMP"
#				}
#			},
#			"essid": "HotelChernoMore",
#			"frequency": {
#				"channel": 11,
#				"unit": "GHz",
#				"value": 2.462
#			},
#			"mac": "F0:9F:C2:91:E3:1D",
#			"signal": {
#				"quality_cur": 52,
#				"quality_max": 70,
#				"signal_level": {
#					"unit": "dBm",
#					"value": -58
#				}
#			}
#		},
#		...
#	]
#
def __get_accesspoints_for_interface(interface:str, c = None, runAsRoot:bool = False) -> list:
	if runAsRoot:
		stdout, _, _x = run(c, "sudo -n /sbin/iwlist " + interface + " scan")
	else:
		stdout, _, _ = run(c, "/sbin/iwlist " + interface + " scan")
	lines = stdout.strip().split("\n")

	"""
	wlp2s0	Scan completed :
			Cell 01 - Address: F0:9F:C2:91:E3:1D
						Channel:11
						Frequency:2.462 GHz (Channel 11)
						Quality=51/70  Signal level=-59 dBm  
						Encryption key:on
						ESSID:"HotelChernoMore"
						Bit Rates:1 Mb/s; 2 Mb/s; 5.5 Mb/s; 11 Mb/s; 6 Mb/s
								9 Mb/s; 12 Mb/s; 18 Mb/s
						Bit Rates:24 Mb/s; 36 Mb/s; 48 Mb/s; 54 Mb/s
						Mode:Master
						Extra:tsf=00000450e91d6016
						Extra: Last beacon: 3816ms ago
						IE: Unknown: 000F486F74656C436865726E6F4D6F7265
						IE: Unknown: 010882848B968C129824
						IE: Unknown: 03010B
						IE: Unknown: 2A0100
						IE: Unknown: 3204B048606C
						IE: Unknown: 0B050500630000
						IE: Unknown: 2D1AAC0103FFFF000000000000000000000100000000000000000000
						IE: Unknown: 3D160B000D00000000000000000000000000000000000000
						IE: Unknown: 7F080000000000000040
						IE: Unknown: DD180050F2020101000003A4000027A4000042435E0062322F00
						IE: Unknown: DD0900037F01010000FF7F
						IE: Unknown: DD0A00037F04010008000A00
						IE: Unknown: DD1300156D00010100010282E58106F09FC290E31D
						IE: IEEE 802.11i/WPA2 Version 1
							Group Cipher : CCMP
							Pairwise Ciphers (1) : CCMP
							Authentication Suites (1) : PSK
			Cell 02 - Address: F2:9F:C2:91:E1:9F
						Channel:1
						Frequency:2.412 GHz (Channel 1)
						Quality=32/70  Signal level=-78 dBm  
						Encryption key:on
						ESSID:""
						Bit Rates:1 Mb/s; 2 Mb/s; 5.5 Mb/s; 11 Mb/s; 6 Mb/s
								9 Mb/s; 12 Mb/s; 18 Mb/s
						Bit Rates:24 Mb/s; 36 Mb/s; 48 Mb/s; 54 Mb/s
						Mode:Master
						Extra:tsf=0000040115180180
						Extra: Last beacon: 20328ms ago
						IE: Unknown: 0000
						IE: Unknown: 010882848B968C129824
						IE: Unknown: 030101
						IE: Unknown: 050402030000
						IE: Unknown: 2A0100
						IE: Unknown: 3204B048606C
						IE: Unknown: 0B050000530000
						IE: Unknown: 2D1AAC0103FFFF000000000000000000000100000000000000000000
						IE: Unknown: 3D1601080C00000000000000000000000000000000000000
						IE: Unknown: 7F080000000000000040
						IE: Unknown: DD180050F2020101000003A4000027A4000042435E0062322F00
						IE: Unknown: DD0900037F01010000FF7F
						IE: Unknown: DD0A00037F04010008000A00
						IE: Unknown: DD1600156D00010128010282E58106F09FC290E19F820100
						IE: IEEE 802.11i/WPA2 Version 1
							Group Cipher : CCMP
							Pairwise Ciphers (1) : CCMP
							Authentication Suites (1) : PSK
			...
			Cell 15 - Address: F0:9F:C2:7E:2D:76
						Channel:36
						Frequency:5.18 GHz (Channel 36)
						Quality=34/70  Signal level=-76 dBm  
						Encryption key:on
						ESSID:"HotelChernoMore"
						Bit Rates:6 Mb/s; 9 Mb/s; 12 Mb/s; 18 Mb/s; 24 Mb/s
								36 Mb/s; 48 Mb/s; 54 Mb/s
						Mode:Master
						Extra:tsf=00000450e502e841
						Extra: Last beacon: 3672ms ago
						IE: Unknown: 000F486F74656C436865726E6F4D6F7265
						IE: Unknown: 01088C129824B048606C
						IE: Unknown: 030124
						IE: Unknown: 200103
						IE: Unknown: 0B050000050000
						IE: Unknown: 2D1AEF091BFFFF000000000000000000000100000000000000000000
						IE: Unknown: 3D1624050D00000000000000000000000000000000000000
						IE: Unknown: 7F080000000000000040
						IE: Unknown: BF0CB2018033FAFF0000FAFF0000
						IE: Unknown: C005000000FCFF
						IE: Unknown: C30402D8D8D8
						IE: Unknown: DD180050F2020101000003A4000027A4000042435E0062322F00
						IE: Unknown: DD0900037F01010000FF7F
						IE: Unknown: DD1300156D00010100010217E58106F09FC27C2D76
						IE: IEEE 802.11i/WPA2 Version 1
							Group Cipher : CCMP
							Pairwise Ciphers (1) : CCMP
							Authentication Suites (1) : PSK
	"""

	assert lines[0].find("Scan completed") > 0
	lines = lines[1:]

	lines = removeAllLeadingSpaces(lines)

	radioCellDict = groupLinesByLeadingSpace(lines)

	byESSID = {}
	withoutESSID = []
	n = 0
	for key, lines in radioCellDict.items():
		# key is something like this: "Cell 01 - Address: F0:9F:C2:91:E3:1D"

		# now process the first line and split it
		pos = key.index(" - ")
		cellKey = key[:pos].strip()
		addressPart = key[pos+3:]

		# append lines starting with spaces to the corresponding previous line
		lines2 = []
		lines3 = []
		lastLine = None
		bAddAll = False
		for line in lines:
			if bAddAll:
				lines3.append(line)
				continue

			if line.startswith("IE: "):
				# stop processing here: these lines come last
				if lastLine is not None:
					lines2.append(lastLine)
					lastLine = None
				bAddAll = True
				continue

			if lastLine is None:
				lastLine = line
			else:
				if line[0].isspace():
					lastLine += line
				else:
					lines2.append(lastLine)
					lastLine = line
		assert lastLine is None

		# group everything together
		d = __parse_single_wifi_cell(interface, [ addressPart ] + lines2, lines3)
		n += 1

		essid = d.get("essid")
		if essid is None:
			withoutESSID.append(d)
		else:
			essidList = byESSID.get(essid)
			if essidList is None:
				essidList = []
				byESSID[essid] = essidList
			essidList.append(d)

	"""
	{
	"0": [
		"Address: 18:E8:29:AB:FA:0C",
		"Channel:36",
		"Frequency:5.18 GHz (Channel 36)",
		"Quality=53/70  Signal level=-57 dBm  ",
		"Encryption key:on",
		"ESSID:\"ConfHall 1+\"",
		"Bit Rates:6 Mb/s; 9 Mb/s; 12 Mb/s; 18 Mb/s; 24 Mb/s	  36 Mb/s; 48 Mb/s; 54 Mb/s",
		"Mode:Master",
		"Extra:tsf=000000380c212954",
		"Extra: Last beacon: 3764ms ago",
		"IE: Unknown: 01088C129824B048606C",
		"IE: Unknown: 030124",
		"IE: Unknown: 200103",
		"IE: Unknown: 0B050700080000",
		"IE: Unknown: 2D1AEF091BFFFF000000000000000000000100000000000000000000",
		"IE: Unknown: 3D16240D0C00000000000000000000000000000000000000",
		"IE: Unknown: 7F080000000000000040",
		"IE: Unknown: BF0CB2018033FAFF0000FAFF0000",
		"IE: Unknown: C005000000FCFF",
		"IE: Unknown: C30402D8D8D8",
		"IE: Unknown: DD180050F2020101000003A4000027A4000042435E0062322F00",
		"IE: Unknown: DD0900037F01010000FF7F",
		"IE: Unknown: DD1300156D00010100010217E5810618E829A9FA0C",
		"IE: IEEE 802.11i/WPA2 Version 1",
		"    Group Cipher : CCMP",
		"    Pairwise Ciphers (1) : CCMP",
		"    Authentication Suites (1) : PSK"
	],
		...,
	"2": [
		"Address: E4:8D:8C:6D:C3:F5",
		"Channel:1",
		"Frequency:2.412 GHz (Channel 1)",
		"Quality=63/70  Signal level=-47 dBm  ",
		"Encryption key:on",
		"ESSID:\"ConfHall 1,2\"",
		"Bit Rates:1 Mb/s; 2 Mb/s; 5.5 Mb/s; 11 Mb/s; 6 Mb/s	  9 Mb/s; 12 Mb/s; 18 Mb/s",
		"Bit Rates:24 Mb/s; 36 Mb/s; 48 Mb/s; 54 Mb/s",
		"Mode:Master",
		"Extra:tsf=000000411f6f43d8",
		"Extra: Last beacon: 4364ms ago",
		"IE: Unknown: 010882848B960C121824",
		"IE: Unknown: 030101",
		"IE: Unknown: 2A0100",
		"IE: Unknown: 2D1A6E1003FFFF000000000000000000000000000000000000000000",
		"IE: IEEE 802.11i/WPA2 Version 1",
		"    Group Cipher : CCMP",
		"    Pairwise Ciphers (2) : CCMP TKIP",
		"    Authentication Suites (1) : PSK",
		"IE: Unknown: 32043048606C",
		"IE: Unknown: 3D1601050000000000000000000000000000000000000000",
		"IE: Unknown: DD2A000C42000000011E0010000004661E060000453438443843364443334635000000000000000005026C09",
		"IE: WPA Version 1",
		"    Group Cipher : CCMP",
		"    Pairwise Ciphers (2) : CCMP TKIP",
		"    Authentication Suites (1) : PSK",
		"IE: Unknown: DD180050F2020101000003A4000027A4000042435E0062322F00",
		"IE: Unknown: DD1E00904C336E1003FFFF000000000000000000000000000000000000000000",
		"IE: Unknown: DD1A00904C3401050000000000000000000000000000000000000000"
	],
		...
	}
	"""

	return {
		"byESSID": byESSID,
		"withoutESSID": withoutESSID
	}
	return radioCellDict
#

#
# Retrieve network information.
#
# Note: this function makes use of <c>get_ifconfig()</c> and adds more data. If you invoke <c>get_net_info()</c> there is no need to invoke <c>get_ifconfig()</c> any more.
#
# Returns:
#	{
#		"enp0s31f6": {
#			"ifname": "enp0s31f6",
#			"is_wlan": false,
#			"mac_addr": "a6:b5:c4:d3:e2:f1",
#			"mtu": 1500
#		},
#		"lo": {
#			"ifname": "lo",
#			"ip4_addr": "127.0.0.1",
#			"ip4_broadcastAddr": null,
#			"ip4_netmask": "255.0.0.0",
#			"ip6_addr": "::1",
#			"ip6_scope": "host",
#			"is_wlan": false,
#			"mac_addr": "00:00:00:00:00:00",
#			"mtu": 65536
#		},
#		"wlp2s0": {
#			"ifname": "wlp2s0",
#			"ip4_addr": "192.168.99.123",
#			"ip4_broadcastAddr": "192.168.99.255",
#			"ip4_netmask": "255.255.255.0",
#			"ip6_addr": "........",
#			"ip6_scope": "link",
#			"is_wlan": true,
#			"mac_addr": "a1:b2:c3:d4:e5:f6",
#			"mtu": 1500
#		}
#	}
#
@cacheCalls(seconds=3, dependArgs=[0, 1])
def get_accesspoints(c = None, runAsRoot:bool = False) -> dict:
	ret = {}

	groups = _get_network_file_list(c)
	interfaces = list(groups.keys())

	ret = {}

	for interface in interfaces:
		filesAndDirs = groups[interface]
		if "wireless" in filesAndDirs:
			ret[interface] = __get_accesspoints_for_interface(interface, c, runAsRoot)

	return ret
#














